"use strict";

var _Graph = _interopRequireDefault(require("../Graph"));

var _GraphVertex = _interopRequireDefault(require("../GraphVertex"));

var _GraphEdge = _interopRequireDefault(require("../GraphEdge"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Graph', () => {
  it('should add vertices to graph', () => {
    const graph = new _Graph.default();
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    graph.addVertex(vertexA).addVertex(vertexB);
    expect(graph.toString()).toBe('A,B');
    expect(graph.getVertexByKey(vertexA.getKey())).toEqual(vertexA);
    expect(graph.getVertexByKey(vertexB.getKey())).toEqual(vertexB);
  });
  it('should add edges to undirected graph', () => {
    const graph = new _Graph.default();
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    graph.addEdge(edgeAB);
    expect(graph.getAllVertices().length).toBe(2);
    expect(graph.getAllVertices()[0]).toEqual(vertexA);
    expect(graph.getAllVertices()[1]).toEqual(vertexB);
    const graphVertexA = graph.findVertexByKey(vertexA.getKey());
    const graphVertexB = graph.findVertexByKey(vertexB.getKey());
    expect(graph.toString()).toBe('A,B');
    expect(graphVertexA).toBeDefined();
    expect(graphVertexB).toBeDefined();
    expect(graph.findVertexByKey('not existing')).toBeNull();
    expect(graphVertexA.getNeighbors().length).toBe(1);
    expect(graphVertexA.getNeighbors()[0]).toEqual(vertexB);
    expect(graphVertexA.getNeighbors()[0]).toEqual(graphVertexB);
    expect(graphVertexB.getNeighbors().length).toBe(1);
    expect(graphVertexB.getNeighbors()[0]).toEqual(vertexA);
    expect(graphVertexB.getNeighbors()[0]).toEqual(graphVertexA);
  });
  it('should add edges to directed graph', () => {
    const graph = new _Graph.default(true);
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    graph.addEdge(edgeAB);
    const graphVertexA = graph.findVertexByKey(vertexA.getKey());
    const graphVertexB = graph.findVertexByKey(vertexB.getKey());
    expect(graph.toString()).toBe('A,B');
    expect(graphVertexA).toBeDefined();
    expect(graphVertexB).toBeDefined();
    expect(graphVertexA.getNeighbors().length).toBe(1);
    expect(graphVertexA.getNeighbors()[0]).toEqual(vertexB);
    expect(graphVertexA.getNeighbors()[0]).toEqual(graphVertexB);
    expect(graphVertexB.getNeighbors().length).toBe(0);
  });
  it('should find edge by vertices in undirected graph', () => {
    const graph = new _Graph.default();
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB, 10);
    graph.addEdge(edgeAB);
    const graphEdgeAB = graph.findEdge(vertexA, vertexB);
    const graphEdgeBA = graph.findEdge(vertexB, vertexA);
    const graphEdgeAC = graph.findEdge(vertexA, vertexC);
    const graphEdgeCA = graph.findEdge(vertexC, vertexA);
    expect(graphEdgeAC).toBeNull();
    expect(graphEdgeCA).toBeNull();
    expect(graphEdgeAB).toEqual(edgeAB);
    expect(graphEdgeBA).toEqual(edgeAB);
    expect(graphEdgeAB.weight).toBe(10);
  });
  it('should find edge by vertices in directed graph', () => {
    const graph = new _Graph.default(true);
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB, 10);
    graph.addEdge(edgeAB);
    const graphEdgeAB = graph.findEdge(vertexA, vertexB);
    const graphEdgeBA = graph.findEdge(vertexB, vertexA);
    const graphEdgeAC = graph.findEdge(vertexA, vertexC);
    const graphEdgeCA = graph.findEdge(vertexC, vertexA);
    expect(graphEdgeAC).toBeNull();
    expect(graphEdgeCA).toBeNull();
    expect(graphEdgeBA).toBeNull();
    expect(graphEdgeAB).toEqual(edgeAB);
    expect(graphEdgeAB.weight).toBe(10);
  });
  it('should return vertex neighbors', () => {
    const graph = new _Graph.default(true);
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeAC = new _GraphEdge.default(vertexA, vertexC);
    graph.addEdge(edgeAB).addEdge(edgeAC);
    const neighbors = graph.getNeighbors(vertexA);
    expect(neighbors.length).toBe(2);
    expect(neighbors[0]).toEqual(vertexB);
    expect(neighbors[1]).toEqual(vertexC);
  });
  it('should throw an error when trying to add edge twice', () => {
    function addSameEdgeTwice() {
      const graph = new _Graph.default(true);
      const vertexA = new _GraphVertex.default('A');
      const vertexB = new _GraphVertex.default('B');
      const edgeAB = new _GraphEdge.default(vertexA, vertexB);
      graph.addEdge(edgeAB).addEdge(edgeAB);
    }

    expect(addSameEdgeTwice).toThrow();
  });
  it('should return the list of all added edges', () => {
    const graph = new _Graph.default(true);
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC);
    graph.addEdge(edgeAB).addEdge(edgeBC);
    const edges = graph.getAllEdges();
    expect(edges.length).toBe(2);
    expect(edges[0]).toEqual(edgeAB);
    expect(edges[1]).toEqual(edgeBC);
  });
  it('should calculate total graph weight for default graph', () => {
    const graph = new _Graph.default();
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const vertexD = new _GraphVertex.default('D');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC);
    const edgeCD = new _GraphEdge.default(vertexC, vertexD);
    const edgeAD = new _GraphEdge.default(vertexA, vertexD);
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeAD);
    expect(graph.getWeight()).toBe(0);
  });
  it('should calculate total graph weight for weighted graph', () => {
    const graph = new _Graph.default();
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const vertexD = new _GraphVertex.default('D');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB, 1);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC, 2);
    const edgeCD = new _GraphEdge.default(vertexC, vertexD, 3);
    const edgeAD = new _GraphEdge.default(vertexA, vertexD, 4);
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeAD);
    expect(graph.getWeight()).toBe(10);
  });
  it('should be possible to delete edges from graph', () => {
    const graph = new _Graph.default();
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC);
    const edgeAC = new _GraphEdge.default(vertexA, vertexC);
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeAC);
    expect(graph.getAllEdges().length).toBe(3);
    graph.deleteEdge(edgeAB);
    expect(graph.getAllEdges().length).toBe(2);
    expect(graph.getAllEdges()[0].getKey()).toBe(edgeBC.getKey());
    expect(graph.getAllEdges()[1].getKey()).toBe(edgeAC.getKey());
  });
  it('should should throw an error when trying to delete not existing edge', () => {
    function deleteNotExistingEdge() {
      const graph = new _Graph.default();
      const vertexA = new _GraphVertex.default('A');
      const vertexB = new _GraphVertex.default('B');
      const vertexC = new _GraphVertex.default('C');
      const edgeAB = new _GraphEdge.default(vertexA, vertexB);
      const edgeBC = new _GraphEdge.default(vertexB, vertexC);
      graph.addEdge(edgeAB);
      graph.deleteEdge(edgeBC);
    }

    expect(deleteNotExistingEdge).toThrowError();
  });
  it('should be possible to reverse graph', () => {
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const vertexD = new _GraphVertex.default('D');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeAC = new _GraphEdge.default(vertexA, vertexC);
    const edgeCD = new _GraphEdge.default(vertexC, vertexD);
    const graph = new _Graph.default(true);
    graph.addEdge(edgeAB).addEdge(edgeAC).addEdge(edgeCD);
    expect(graph.toString()).toBe('A,B,C,D');
    expect(graph.getAllEdges().length).toBe(3);
    expect(graph.getNeighbors(vertexA).length).toBe(2);
    expect(graph.getNeighbors(vertexA)[0].getKey()).toBe(vertexB.getKey());
    expect(graph.getNeighbors(vertexA)[1].getKey()).toBe(vertexC.getKey());
    expect(graph.getNeighbors(vertexB).length).toBe(0);
    expect(graph.getNeighbors(vertexC).length).toBe(1);
    expect(graph.getNeighbors(vertexC)[0].getKey()).toBe(vertexD.getKey());
    expect(graph.getNeighbors(vertexD).length).toBe(0);
    graph.reverse();
    expect(graph.toString()).toBe('A,B,C,D');
    expect(graph.getAllEdges().length).toBe(3);
    expect(graph.getNeighbors(vertexA).length).toBe(0);
    expect(graph.getNeighbors(vertexB).length).toBe(1);
    expect(graph.getNeighbors(vertexB)[0].getKey()).toBe(vertexA.getKey());
    expect(graph.getNeighbors(vertexC).length).toBe(1);
    expect(graph.getNeighbors(vertexC)[0].getKey()).toBe(vertexA.getKey());
    expect(graph.getNeighbors(vertexD).length).toBe(1);
    expect(graph.getNeighbors(vertexD)[0].getKey()).toBe(vertexC.getKey());
  });
  it('should return vertices indices', () => {
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const vertexD = new _GraphVertex.default('D');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC);
    const edgeCD = new _GraphEdge.default(vertexC, vertexD);
    const edgeBD = new _GraphEdge.default(vertexB, vertexD);
    const graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeBD);
    const verticesIndices = graph.getVerticesIndices();
    expect(verticesIndices).toEqual({
      A: 0,
      B: 1,
      C: 2,
      D: 3
    });
  });
  it('should generate adjacency matrix for undirected graph', () => {
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const vertexD = new _GraphVertex.default('D');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC);
    const edgeCD = new _GraphEdge.default(vertexC, vertexD);
    const edgeBD = new _GraphEdge.default(vertexB, vertexD);
    const graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeBD);
    const adjacencyMatrix = graph.getAdjacencyMatrix();
    expect(adjacencyMatrix).toEqual([[Infinity, 0, Infinity, Infinity], [0, Infinity, 0, 0], [Infinity, 0, Infinity, 0], [Infinity, 0, 0, Infinity]]);
  });
  it('should generate adjacency matrix for directed graph', () => {
    const vertexA = new _GraphVertex.default('A');
    const vertexB = new _GraphVertex.default('B');
    const vertexC = new _GraphVertex.default('C');
    const vertexD = new _GraphVertex.default('D');
    const edgeAB = new _GraphEdge.default(vertexA, vertexB, 2);
    const edgeBC = new _GraphEdge.default(vertexB, vertexC, 1);
    const edgeCD = new _GraphEdge.default(vertexC, vertexD, 5);
    const edgeBD = new _GraphEdge.default(vertexB, vertexD, 7);
    const graph = new _Graph.default(true);
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeBD);
    const adjacencyMatrix = graph.getAdjacencyMatrix();
    expect(adjacencyMatrix).toEqual([[Infinity, 2, Infinity, Infinity], [Infinity, Infinity, 1, 7], [Infinity, Infinity, Infinity, 5], [Infinity, Infinity, Infinity, Infinity]]);
  });
});